package configuration

import (
	"encoding/json"
	yaml "gopkg.in/yaml.v3"
	"strings"
	"testing"
)

func TestBoolValueString(t *testing.T) {
	tests := []struct {
		input    string
		expected string
	}{
		{"true", "true"},
		{"false", "false"},
		{"one", "one"},
		{"", ""},
		{" ", " "},
		{"off", "false"},
		{"on", "true"},
	}

	for _, tt := range tests {
		t.Run(string(tt.input), func(t *testing.T) {
			bv, _ := NewBoolValue(tt.input)

			result := bv.String()
			if result != tt.expected {
				t.Errorf("result = %s; expected %s", result, tt.expected)
			}
		})
	}
}
func TestBoolValueRaw(t *testing.T) {
	tests := []struct {
		input    string
		expected string
	}{
		{"true", "true"},
		{"false", "false"},
		{"one", "one"},
		{"", ""},
		{" ", " "},
		{"off", "off"},
		{"on", "on"},
	}

	for _, tt := range tests {
		t.Run(string(tt.input), func(t *testing.T) {
			bv, _ := NewBoolValue(tt.input)

			result := bv.Raw()
			if result != tt.expected {
				t.Errorf("result = %s; expected %s", result, tt.expected)
			}
		})
	}
}

func TestBoolValueIsTrue(t *testing.T) {
	tests := []struct {
		input    string
		expected bool
	}{
		{"true", true},
		{"false", false},
		{"one", false},
		{"", false},
		{" ", false},
		{"gurke", false},
		{"off", false},
		{"on", true},
		{"no", false},
		{"yes", true},
		{"0", false},
		{"1", true},
	}

	for _, tt := range tests {
		t.Run(string(tt.input), func(t *testing.T) {
			bv, _ := NewBoolValue(tt.input)

			result := bv.IsTrue()
			if result != tt.expected {
				t.Errorf("result = %t; expected %t", result, tt.expected)
			}
		})
	}
}

// TestBoolValueYAMLSerialization tests the YAML serialization and deserialization of BoolValue
func TestBoolValueYAMLMarshaling(t *testing.T) {
	cases := []struct {
		name       string
		boolValue  BoolValue
		expected   string
		shouldFail bool
	}{
		{
			name:      "True value",
			boolValue: BoolValue{value: true, valid: true, raw: "true"},
			expected:  "true",
		},
		{
			name:      "False value",
			boolValue: BoolValue{value: false, valid: true, raw: "false"},
			expected:  "false",
		},
		{
			name:      "Invalid value",
			boolValue: BoolValue{valid: false, raw: "maybe"},
			expected:  "false",
		},
	}

	for _, c := range cases {
		t.Run(c.name, func(t *testing.T) {
			data, err := yaml.Marshal(&c.boolValue)
			if err != nil {
				if !c.shouldFail {
					t.Errorf("Marshaling failed when it should not have: %v", err)
				}
				return
			}

			if strings.TrimSpace(string(data)) != c.expected {
				t.Errorf("Expected %s but got %s", c.expected, string(data))
			}
		})
	}
}

func TestBoolValueYAMLUnmarshaling(t *testing.T) {
	cases := []struct {
		name        string
		yamlContent string
		expected    string
		shouldFail  bool
	}{
		{
			name:        "True value",
			yamlContent: `true`,
			expected:    "true",
		},
		{
			name:        "False value",
			yamlContent: `false`,
			expected:    "false",
		},
		{
			name:        "Invalid value",
			yamlContent: `maybe`,
			shouldFail:  true,
		},
	}

	for _, c := range cases {
		t.Run(c.name, func(t *testing.T) {

			var unmarshal BoolValue
			data := []byte(c.yamlContent)
			err := yaml.Unmarshal(data, &unmarshal)
			if err != nil {
				if !c.shouldFail {
					t.Errorf("Unmarshaling failed when it should not have: %v", err)
				}
				return
			}
			if unmarshal.String() != c.expected {
				t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, c.expected)
			}
		})
	}
}

type BoolValueTestStruct struct {
	Value BoolValue
}

func TestBoolValueJsonDecode(t *testing.T) {

	cases := []struct {
		name        string
		jsonContent string
		expected    string
		shouldFail  bool
	}{
		{
			name:        "True value",
			jsonContent: "{\"value\":\"true\"}",
			expected:    "true",
		},
		{
			name:        "False value",
			jsonContent: "{\"value\":\"false\"}",
			expected:    "false",
		},
		{
			name:        "Invalid value",
			jsonContent: "{\"value\":\"gurke\"}",
			shouldFail:  true,
		},
	}

	for _, c := range cases {
		t.Run(c.name, func(t *testing.T) {

			var unmarshal BoolValueTestStruct
			data := []byte(c.jsonContent)
			err := json.Unmarshal(data, &unmarshal)
			if err != nil {
				if !c.shouldFail {
					t.Errorf("Unmarshaling failed when it should not have: %v", err)
				}
				return
			}
			if unmarshal.Value.String() != c.expected {
				t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, c.expected)
			}
		})
	}
}

func TestBoolValueYamlDecode2(t *testing.T) {
	yamlContent := `
value1: true
value2: false
value3: on
value4: 1
`
	var unmarshal map[string]BoolValue
	data := []byte(yamlContent)
	err := yaml.Unmarshal(data, &unmarshal)
	if err != nil {
		t.Errorf("Unmarshaling failed when it should not have: %v", err)
		return
	}
	if unmarshal["value1"].String() != "true" {
		t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, "true")
	}
	if unmarshal["value2"].String() != "false" {
		t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, "false")
	}
	if unmarshal["value3"].String() != "true" {
		t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, "true")
	}
	if unmarshal["value4"].String() != "true" {
		t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, "true")
	}

}

func TestBoolValueYamlDecode(t *testing.T) {

	cases := []struct {
		name        string
		yamlContent string
		expected    string
		shouldFail  bool
	}{
		{
			name: "True value",
			yamlContent: `
value: true
`,
			expected: "true",
		},
		{
			name: "False value",
			yamlContent: `
value: false
`,
			expected: "false",
		},
		{
			name: "Invalid value",
			yamlContent: `
value: gurke
`,
			shouldFail: true,
		},
	}

	for _, c := range cases {
		t.Run(c.name, func(t *testing.T) {

			var unmarshal BoolValueTestStruct
			data := []byte(c.yamlContent)
			err := yaml.Unmarshal(data, &unmarshal)
			if err != nil {
				if !c.shouldFail {
					t.Errorf("Unmarshaling failed when it should not have: %v", err)
				}
				return
			}
			if unmarshal.Value.String() != c.expected {
				t.Errorf("Unmarshaled value %v does not match original %v", unmarshal, c.expected)
			}
		})
	}
}
